#!/usr/bin/env python3
import os

from setuptools import setup

BASEDIR = os.path.abspath(os.path.dirname(__file__))


def get_version():
    """Find the version of the package"""
    version = None
    version_file = os.path.join(BASEDIR, "ovos_tts_plugin_server", "version.py")
    major, minor, build, alpha = (None, None, None, None)
    with open(version_file) as f:
        for line in f:
            if "VERSION_MAJOR" in line:
                major = line.split("=")[1].strip()
            elif "VERSION_MINOR" in line:
                minor = line.split("=")[1].strip()
            elif "VERSION_BUILD" in line:
                build = line.split("=")[1].strip()
            elif "VERSION_ALPHA" in line:
                alpha = line.split("=")[1].strip()

            if (major and minor and build and alpha) or "# END_VERSION_BLOCK" in line:
                break
    version = f"{major}.{minor}.{build}"
    if alpha and int(alpha) > 0:
        version += f"a{alpha}"
    return version


def required(requirements_file):
    """Read requirements file and remove comments and empty lines."""
    with open(os.path.join(BASEDIR, requirements_file), "r") as f:
        requirements = f.read().splitlines()
        if "MYCROFT_LOOSE_REQUIREMENTS" in os.environ:
            print("USING LOOSE REQUIREMENTS!")
            requirements = [r.replace("==", ">=").replace("~=", ">=") for r in requirements]
        return [pkg for pkg in requirements if pkg.strip() and not pkg.startswith("#")]


def get_description():
    with open(os.path.join(BASEDIR, "readme.md"), "r") as f:
        long_description = f.read()
    return long_description


PLUGIN_ENTRY_POINT = "ovos-tts-plugin-server = ovos_tts_plugin_server:OVOSServerTTS"
CONFIG_ENTRY_POINT = "ovos-tts-plugin-server.config = ovos_tts_plugin_server:OVOSServerTTSConfig"


setup(
    name="ovos-tts-plugin-server",
    version=get_version(),
    description="ovos tts server plugin for mycroft",
    long_description=get_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/OpenVoiceOS/ovos-tts-server-plugin",
    author="JarbasAi",
    author_email="jarbasai@mailfence.com",
    license="Apache-2.0",
    packages=["ovos_tts_plugin_server"],
    install_requires=required("requirements.txt"),
    zip_safe=True,
    include_package_data=True,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Text Processing :: Linguistic",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    keywords="mycroft OpenVoiceOS OVOS plugin tts",
    entry_points={
        "mycroft.plugin.tts": PLUGIN_ENTRY_POINT,
        "mycroft.plugin.tts.config": CONFIG_ENTRY_POINT,
    },
)



"""OpenVoiceOS companion plugin for OpenVoiceOS TTS Server."""
import random
from typing import Any, Dict, List, Optional, Tuple

import requests
from ovos_plugin_manager.templates.tts import TTS, RemoteTTSException, TTSValidator
from ovos_utils.log import LOG

PUBLIC_TTS_SERVERS = [
    "https://pipertts.ziggyai.online",
    "https://tts.smartgic.io/piper",
]


class OVOSServerTTS(TTS):
    """Interface to OVOS TTS server"""

    public_servers: List[str] = PUBLIC_TTS_SERVERS

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, audio_ext="wav", validator=OVOSServerTTSValidator(self))
        self.log = LOG
        if not self.verify_ssl:
            self.log.warning(
                "SSL verification disabled, this is not secure and should"
                "only be used for test systems! Please set up a valid certificate!"
            )

    @property
    def host(self) -> Optional[str]:
        """If using a custom server, set the host here, otherwise it defaults to public servers."""
        return self.config.get("host", None)

    @property
    def v2(self) -> bool:
        """If using default public servers, default to v2, else v1"""
        return self.config.get("v2", self.host is None)

    @property
    def verify_ssl(self) -> bool:
        """Whether or not to verify SSL certificates when connecting to the server. Defaults to True."""
        return self.config.get("verify_ssl", True)

    def get_tts(
        self,
        sentence,
        wav_file,
        lang: Optional[str] = None,
        voice: Optional[str] = None,
    ) -> Tuple[Any, None]:
        """Fetch TTS audio using OVOS TTS server.
        Language and voice can be overridden, otherwise defaults to config."""
        params: Dict[str, Optional[str]] = {
            "lang": lang or self.lang,
            "voice": voice or self.voice,
        }
        if not voice or voice == "default":
            params.pop("voice")
        if self.host:
            if isinstance(self.host, str):
                servers: List[str] = [self.host]
            else:
                servers = self.host
        else:
            servers = self.public_servers
        data: bytes = self._fetch_audio_data(params, sentence, servers)
        self._write_audio_file(wav_file, data)
        return wav_file, None

    def _write_audio_file(self, wav_file: str, data: bytes) -> None:
        with open(file=wav_file, mode="wb") as f:
            f.write(data)

    def _fetch_audio_data(self, params: dict, sentence: str, servers: list) -> bytes:
        """Get audio bytes from servers."""
        random.shuffle(servers)  # Spread the load among all public servers
        for url in servers:
            try:
                if self.v2:
                    url = f"{url}/v2/synthesize"
                    params["utterance"] = sentence
                else:
                    url = f"{url}/synthesize/{sentence}"
                r: requests.Response = requests.get(url=url, params=params, verify=self.verify_ssl, timeout=30)
                if r.ok:
                    return r.content
                self.log.error(f"Failed to get audio, response from {url}: {r.text}")
            except Exception as err:  # pylint: disable=broad-except
                self.log.error(f"Failed to get audio from {url}: {err}")
                continue
        raise RemoteTTSException("All OVOS TTS servers are down!")


class OVOSServerTTSValidator(TTSValidator):
    """Validate settings for OVOS TTS server plugin."""

    def __init__(self, tts) -> None:  # pylint: disable=useless-parent-delegation
        super(OVOSServerTTSValidator, self).__init__(tts)

    def validate_lang(self) -> None:
        """Validate language setting."""
        return

    def validate_connection(self) -> None:
        """Validate connection to server."""
        return

    def get_tts_class(self):
        """Return TTS class."""
        return OVOSServerTTS


OVOSServerTTSConfig: Dict[Any, Any] = {}
